use crate::station::Station;
use axum::{
    extract::State,
    response::{IntoResponse, Redirect},
    routing::{get, post},
    Form, Router,
};
use maud::{html, Markup};
use serde::Deserialize;
use sqlx::SqlitePool;
use std::sync::Arc;
use tower_sessions::Session;

#[derive(Deserialize)]
struct CreateForm {
    name: String,
}

async fn create(
    State(db): State<Arc<SqlitePool>>,
    session: Session,
    Form(form): Form<CreateForm>,
) -> impl IntoResponse {
    match Station::create(&db, &form.name).await{
Ok(_) => 
    session
        .insert(
            "succ",
            &format!("Station '{}' erfolgreich erstellt!", form.name),
        )
        .await
        .unwrap(),
    Err(e) => 
    session
        .insert(
            "err",
            &format!("Station '{}' konnte _NICHT_ erstellt werden, da es bereits eine Station mit diesem Namen gibt ({e})!", form.name),
        )
        .await
        .unwrap(),

    }

    Redirect::to("/station")
}

async fn delete(
    State(db): State<Arc<SqlitePool>>,
    session: Session,
    axum::extract::Path(id): axum::extract::Path<i64>,
) -> impl IntoResponse {
    let Some(station) = Station::find_by_id(&db, id).await else {
        session
            .insert(
                "err",
                &format!(
                    "Station mit ID {id} konnte nicht gelöscht werden, da sie nicht existiert"
                ),
            )
            .await
            .unwrap();

        return Redirect::to("/station");
    };

    station.delete(&db).await;

    session
        .insert(
            "succ",
            &format!("Station '{}' erfolgreich gelöscht!", station.name),
        )
        .await
        .unwrap();

    Redirect::to("/station")
}

async fn index(State(db): State<Arc<SqlitePool>>, session: Session) -> Markup {
    let stations = Station::all(&db).await;

    // Get and clear flash message
    let succ_msg = session.get::<String>("succ").await.unwrap_or(None);
    if succ_msg.is_some() {
        session.remove::<String>("succ").await.unwrap();
    }
    let err_msg = session.get::<String>("err").await.unwrap_or(None);
    if err_msg.is_some() {
        session.remove::<String>("err").await.unwrap();
    }

    html! {
        h1 { "Stationen" }
        @if let Some(message) = succ_msg {
            div class="alert alert-success" {
                (message)
            }
        }
        @if let Some(message) = err_msg {
            div class="alert alert-success" {
                (message)
            }
        }
        ol {
            @for station in &stations {
                li {
                    (station.name)
                     a href=(format!("/station/{}/delete", station.id))
                       onclick="return confirm('Bist du sicher, dass die Station gelöscht werden soll? Das kann _NICHT_ mehr rückgängig gemacht werden.');" {
                         "🗑️"
                     }
                }
            }
        }
        h2 { "Neue Station" }
        form action="/station" method="post" {
            div {
                label for="name" { "Name:" }
                input type="text" name="name" required;
            }
            div {
                button type="submit" { "Submit" }
            }
        }
    }
}

pub(super) fn routes() -> Router<Arc<SqlitePool>> {
    Router::new()
        .route("/", get(index))
        .route("/{id}/delete", get(delete))
        .route("/", post(create))
}
